#######################################################################################################
# File-Name:       SimulatingAccuracyMeasures.R           
# Date:            10.9.2014                                
# Author:          (Nigel) Benjamin Neely (nigelneely@gmail.com)
# Purpose:         To simulate data under controlled assumptions to ensure that our variance estimator
#                  is working as it should be.
# Data Used:       N/A
# Packages Used:   N/A
#######################################################################################################
# NOTE: If you are running this in the R console you must use the 'setwd' command to set the 
# working directory for the console to whereever you have saved this file prior to running.
# Otherwise you will see errors when loading data or saving figures!
#######################################################################################################
generateDataBinaryClassification <- function(TPF,FPF,PPV,NPV,FOF,FDF,populationSize) {
  if (sum(c(hasArg(TPF),hasArg(FPF),hasArg(PPV),hasArg(NPV),hasArg(FOF),hasArg(FDF)))>=4) {
    stop("0 DOF, cannot guantee data generation will provide the requested attributes")
  }
  #Instantiate a table that could be used for display purposes later on (ensure this is exposed in the return of fucntion)-------------------------------
  table           <- matrix(nrow=2,ncol=2)
  colnames(table) <- c('Disease Positive','Disease Negative')
  rownames(table) <- c('Test Positive','Test Negative')

#Need to set ourselves up for solving a system of linear equations:------------------------------------------------------------------------------------
#First, what cells do we know we need?
  LL        <- as.list(c(a=NA,     b=NA,c=NA,d=NA))
  sysEqn    <- as.list(c(order1=NA,order2=NA,order3=NA,order4=NA,order5=NA,order6=NA))
  response  <- as.list(c(resp1=NA, resp2=NA, resp3=NA, resp4=NA,resp5=NA,resp6=NA))
  if (hasArg(TPF)) {
    LL[1]       <- "A"
    LL[3]       <- "C"
    sysEqn[1]   <- paste0(TPF-1,",0,",TPF,",0") 
    response[1] <- 0
  }
  if (hasArg(FPF)) {
    LL[2]       <- "B"
    LL[4]       <- "D"
    sysEqn[2]   <- paste0("0," , FPF-1 , ",0," , FPF)
    response[2] <- 0
  }
  if (hasArg(PPV)) {
    LL[1]       <- "A"
    LL[2]       <- "B"
    sysEqn[3]   <- paste0(PPV-1 , "," , PPV , ",0,0")
    response[3] <- 0
  }
  if (hasArg(NPV)) {
    LL[3]       <- "C"
    LL[4]       <- "D"
    sysEqn[4]   <- paste0("0,0," , NPV , "," , NPV-1)
    response[4] <- 0
  }
  if (hasArg(FOF)) {
    LL[3]       <- "C"
    LL[4]       <- "D"
    sysEqn[5]   <- paste0("0,0," , FOF-1 , "," , FOF)
    response[5] <- 0
  }
  if (hasArg(FDF)) {
    LL[1]       <- "A"
    LL[2]       <- "B"
    sysEqn[6]   <- paste0(FDF , "," , FDF-1 , ",0,0")
    response[6] <- 0
  }

#If we don't have enough equations to satisfy the square matrix condition, we add one which constrains the overall sample------------------------------------
  if ((sum(!is.na(LL))-sum(!is.na(sysEqn)))==1) {
    sysEqn[7]   <- "1,1,1,1"
    names(sysEqn[7]) <- "order7" 
    response[7] <- populationSize
  }

#Now we need to elminate indices that are not needed so that we can correctly configure our system of equations to find a solution---------------------------
  response     <- response[which(!is.na(sysEqn))]
  sysEqn       <- sysEqn[which(!is.na(sysEqn))]
  sysEqn       <- lapply(sysEqn, function(x) paste0(unlist(strsplit(as.character(x), split = ","))[which(!is.na(LL))],collapse=","))
#Solving the system of linear equations created
  A            <- apply(matrix(unlist(lapply(sysEqn, function(x) unlist(strsplit(as.character(x), split = ",")))),ncol=sum(!is.na(LL)),nrow=sum(!is.na(LL))),1,as.numeric)
  b            <- unlist(response)
  answer       <- round(solve(A,b),digits=0)
  names(answer)<- LL[which(!is.na(LL))]
#Create the table used as an attribute to this object (useful for higher view point)-------------------------------------------------------------------------
  rep.row<-function(x,n){
    matrix(rep(x,each=n),nrow=n)
  }

  dat <- data.frame("R"=as.numeric(),"D"=as.numeric())
  for (i in 1:length(LL)) {
    if (names(LL[i])=="a") {
      if (sum(names(answer)=="A")>0) { table[1,1] <- answer[which(names(answer)=="A")]; dat <- rbind(dat,rep.row(c(1,1),answer[which(names(answer)=="A")])) }
      else { table[1,1] <- 0 }
    }
    if (names(LL[i])=="b") {
      if (sum(names(answer)=="B")>0) { table[1,2] <- answer[which(names(answer)=="B")]; dat <- rbind(dat,rep.row(c(1,0),answer[which(names(answer)=="B")])) }
      else { table[1,2] <- 0 }
    }
    if (names(LL[i])=="c") {
      if (sum(names(answer)=="C")>0) { table[2,1] <- answer[which(names(answer)=="C")]; dat <- rbind(dat,rep.row(c(0,1),answer[which(names(answer)=="C")])) }
      else { table[2,1] <- 0 }
    }
    if (names(LL[i])=="d") {
      if (sum(names(answer)=="D")>0) { table[2,2] <- answer[which(names(answer)=="D")]; dat <- rbind(dat,rep.row(c(0,0),answer[which(names(answer)=="D")])) }
      else { table[2,2] <- 0 }
    }
  }
  colnames(dat) <- c("Test","Disease")
  out         <- list(dat = dat,
                      table = table
  )
  return(out)
}

# Example Call
#Typical Call to generateDataBinaryClassification (for the needs of Begg and Greenes implementation)
# a <- generateDataBinaryClassification(TPF=0.8,PPV=0.0,FOF=0.8,populationSize=150)
  


